/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.action;

import com.intellij.lang.StdLanguages;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.fileTypes.FileType;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.*;
import net.sourceforge.intellime.ui.diagram.DiagramFileEditor;
import net.sourceforge.intellime.ui.diagram.Edge;
import net.sourceforge.intellime.ui.diagram.Node;
import net.sourceforge.intellime.ui.element.J2MEElement;
import net.sourceforge.intellime.ui.element.MIDletElement;
import net.sourceforge.intellime.ui.element.NestedListElement;
import net.sourceforge.intellime.ui.element.command.CommandElement;
import net.sourceforge.intellime.ui.element.screen.AlertElement;
import net.sourceforge.intellime.util.*;

import java.io.IOException;
import java.util.*;

public class CreateMIDletFromDiagram extends AnAction {

    private final String FIRST_PARAMETER = "${1}";
    private final String SECOND_PARAMETER = "${2}";
    private final String EXIT_METHOD = "exitMIDlet();";
    private final String SET_CURRENT_ALERT_METHOD = "getDisplay().setCurrent(get_" + FIRST_PARAMETER + "(), get_" + SECOND_PARAMETER + "());";
    private final String SET_CURRENT_DEFAULT_METHOD = "getDisplay().setCurrent(get_" + FIRST_PARAMETER + "());";

    public void update(AnActionEvent e) {
        Project project = (Project) e.getDataContext().getData(DataConstants.PROJECT);
        e.getPresentation().setVisible(true);
        if (project == null) {
            e.getPresentation().setEnabled(false);
        } else {
            e.getPresentation().setEnabled(getDiagramFileEditor(project) != null);
        }
    }

    public void actionPerformed(AnActionEvent e) {
        Project project = (Project) e.getDataContext().getData(DataConstants.PROJECT);
        DiagramFileEditor fileEditor = getDiagramFileEditor(project);
        if (fileEditor != null) {
            createMIDlet(project, fileEditor);
        }
    }

    private DiagramFileEditor getDiagramFileEditor(Project project) {
        FileEditor[] editors = FileEditorManager.getInstance(project).getSelectedEditors();
        DiagramFileEditor fileEditor = null;
        for (int i = 0; i < editors.length && fileEditor == null; i++) {
            if (editors[i].isValid() && editors[i] instanceof DiagramFileEditor) {
                fileEditor = (DiagramFileEditor) editors[i];
            }
        }
        return fileEditor;
    }

    private void createMIDlet(final Project project, DiagramFileEditor fileEditor) {
        final VirtualFile virtualFile = fileEditor.getVirtualFile();
        String packageName = getPackageName(project, virtualFile);
        try {
            final Template t = new Template(Constants.DIAGRAM_TEMPLATE);

            t.put(Constants.PACKAGE_NAME_KEY, packageName);
            t.put(Constants.NAME_KEY, virtualFile.getNameWithoutExtension());

            List<J2MEElement> attributes = getAttributes(fileEditor);
            t.put(Constants.ATTRIBUTES_KEY, attributes);
            t.put(Constants.METHODS_KEY, getMethods(attributes));
            t.put(Constants.INITIALIZE_METHOD_KEY, getInitializeMethod(fileEditor));
            t.put(Constants.ACTIONS_KEY, getActions(fileEditor));

            RunnableHelper.runWriteCommand(project, new Runnable() {
                public void run() {
                    try {
                        t.evaluate(getVirtualFileMidlet(project, virtualFile));
                    } catch (IOException e1) {
                        System.out.println(e1.getMessage());
                    } catch (Template.TemplateException e1) {
                        System.out.println(e1.getMessage());
                    }
                }
            });
        } catch (Exception e1) {
            System.out.println(e1.getMessage());
        }
    }

    private String getPackageName(Project project, VirtualFile virtualFile) {
        String packageName = "";
        PsiFile psiFile = PsiManager.getInstance(project).findFile(virtualFile);
        if (psiFile != null) {
            PsiElement parent = psiFile.getParent();
            if (parent instanceof PsiDirectory) {
                PsiDirectory psiDirectory = (PsiDirectory) parent;
                PsiPackage psiPackage = psiDirectory.getPackage();
                if (psiPackage != null) {
                    packageName = psiPackage.getQualifiedName();
                }
            }
        }
        return packageName;
    }

    private List<J2MEElement> getAttributes(DiagramFileEditor fileEditor) {
        List<J2MEElement> attributes = new ArrayList<J2MEElement>();
        Collection<Node> nodes = fileEditor.getNodes();
        for (Node node : nodes) {
            if (!(node.getScreenElement() instanceof MIDletElement)) {
                attributes.add(node.getScreenElement());
                attributes.addAll(node.getElements());
                attributes.removeAll(node.getNestedListElements());
            }
        }
        return attributes;
    }

    private List<String> getMethods(List<J2MEElement> attributes) {
        List<String> methods = new ArrayList<String>();
        for (J2MEElement element : attributes) {
            methods.add(MidletMethodFactory.getInstance().getMidletMethod(element));
        }
        return methods;
    }

    private String getInitializeMethod(DiagramFileEditor fileEditor) {
        String initializeMethod = "";
        for (Edge edge : fileEditor.getEdges()) {
            Node sourceNode = edge.getSource();
            Node targetNode = edge.getTarget();
            Edge linkedEdge = targetNode.getLinkedEdge(edge);
            if (sourceNode.getScreenElement() instanceof MIDletElement) {
                initializeMethod = getMethod(targetNode, linkedEdge);
                break;
            }
        }
        return initializeMethod;
    }

    private List<String> getActions(DiagramFileEditor fileEditor) throws IOException, Template.TemplateException {
        List<String> actions = new ArrayList<String>();
        for (Edge edge : fileEditor.getEdges()) {
            Node sourceNode = edge.getSource();
            CommandElement command = edge.getCommand();
            Node targetNode = edge.getTarget();
            Edge linkedEdge = targetNode.getLinkedEdge(edge);
            if ((!(sourceNode.getScreenElement() instanceof MIDletElement)) && (command != null)) {
                Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(Constants.DISPLAYABLE_KEY, sourceNode.getProperty());
                parameters.put(Constants.COMMAND_KEY, command.getProperty());
                parameters.put(Constants.METHOD_KEY, getMethod(targetNode, linkedEdge));
                actions.add(Utils.getTemplateEvaluateAsString(Constants.COMMAND_ACTION_TEMPLATE, parameters));
            }
        }
        for (Node node : fileEditor.getNodes()) {
            List<NestedListElement> nestedListElements = node.getNestedListElements();
            if (nestedListElements.size() > 0) {
                Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(Constants.LIST_KEY, node.getScreenElement());
                actions.add(Utils.getTemplateEvaluateAsString(Constants.NESTED_LIST_ACTION_TEMPLATE, parameters));
            }
        }
        return actions;
    }

    private String getMethod(Node targetNode, Edge linkedEdge) {
        String method;
        if (targetNode.getScreenElement() instanceof MIDletElement) {
            method = EXIT_METHOD;
        } else if ((targetNode.getScreenElement() instanceof AlertElement) && (linkedEdge != null)) {
            method = SET_CURRENT_ALERT_METHOD;
            method = method.replace(FIRST_PARAMETER, targetNode.getProperty());
            method = method.replace(SECOND_PARAMETER, linkedEdge.getTarget().getProperty());
        } else {
            method = SET_CURRENT_DEFAULT_METHOD;
            method = method.replace(FIRST_PARAMETER, targetNode.getProperty());
        }
        return method;
    }

    private VirtualFile getVirtualFileMidlet(Project project, VirtualFile virtualFile) {
        VirtualFile virtualFileMidlet = null;
        FileType fileType = StdLanguages.JAVA.getAssociatedFileType();
        if (fileType != null) {
            final String midletFileName = virtualFile.getNameWithoutExtension() + "." + fileType.getDefaultExtension();
            final VirtualFile directory = virtualFile.getParent();
            if (directory != null) {
                virtualFileMidlet = directory.findChild(midletFileName);
                if (virtualFileMidlet == null) {
                    RunnableHelper.runWriteCommand(project, new Runnable() {
                        public void run() {
                            try {
                                directory.createChildData(this, midletFileName);
                            } catch (IOException e) {
                                System.out.println(e.getMessage());
                            }
                        }
                    });
                    virtualFileMidlet = directory.findChild(midletFileName);
                }
            }
        }
        return virtualFileMidlet;
    }

}
