/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.ui.diagram;

import com.intellij.openapi.graph.GraphManager;
import com.intellij.openapi.graph.base.Graph;
import com.intellij.openapi.graph.builder.GraphBuilder;
import com.intellij.openapi.graph.builder.GraphDataModel;
import com.intellij.openapi.graph.builder.util.GraphWrapperUtil;
import com.intellij.openapi.graph.layout.orthogonal.DirectedOrthogonalLayouter;
import com.intellij.openapi.graph.settings.GraphSettingsProvider;
import com.intellij.openapi.graph.view.Graph2D;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vfs.VirtualFile;
import net.sourceforge.intellime.J2MEBundle;
import net.sourceforge.intellime.ui.diagram.helper.DeserializerException;
import net.sourceforge.intellime.ui.diagram.helper.DiagramDeserializer;
import net.sourceforge.intellime.ui.diagram.helper.DiagramSerializer;
import net.sourceforge.intellime.ui.editor.EdgeEditor;
import net.sourceforge.intellime.ui.editor.EdgeEditorFactory;
import net.sourceforge.intellime.ui.editor.EditorDialogBase;
import net.sourceforge.intellime.ui.element.J2MEElement;
import net.sourceforge.intellime.ui.element.command.CommandElement;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class DiagramGraphDataModel extends GraphDataModel<Node, Edge> {

    private List<Node> nodes = new ArrayList<Node>();
    private List<Edge> edges = new ArrayList<Edge>();
    private GraphBuilder graphBuilder;
    private DiagramSerializer serializer;
    private Project project;
    private VirtualFile file;

    public DiagramGraphDataModel(Project project, VirtualFile file) {
        this.project = project;
        this.file = file;
        serializer = new DiagramSerializer(project, file);
        try {
            new DiagramDeserializer(this).buildGraph();
            doLayout();
        } catch (DeserializerException e) {
            Messages.showErrorDialog(
                    J2MEBundle.message("ui.diagram.DiagramDeserializer.invalidFile"),
                    J2MEBundle.message("ui.diagram.DiagramDeserializer.invalidFileTitle")
            );
        }
    }

    public void setGraphBuilder(GraphBuilder graphBuilder) {
        this.graphBuilder = graphBuilder;
    }

    public Project getProject() {
        return project;
    }

    public VirtualFile getFile() {
        return file;
    }

    private void configureEdge(Edge edge) {
        EdgeEditor editor = EdgeEditorFactory.getEditor(this, edge);
        EditorDialogBase<Edge> dialog = editor.getEditorDialog();
        if (dialog != null) {
            // edge edition
            dialog.show();
            if (dialog.isOK()) {
                editor.doEdit(edge);
            } else {
                removeEdge(edge, true);
            }
        } else {
            editor.doEdit(edge);
        }
    }

    public Edge createEdge(Node source, Node target) {
        return createEdge(source, target, true);
    }

    public Edge createEdge(Node source, Node target, boolean serialize) {
        Edge edge = new Edge(source, target);
        edges.add(edge);

        // configure edge
        if (serialize) {
            configureEdge(edge);
        }

        if (graphBuilder != null) {
            graphBuilder.updateGraph();
        }

        if (serialize) {
            serializer.serialize(nodes, edges);
        }

        return edge;
    }

    @NotNull
    public String getEdgeName(Edge edge) {
        String edgeName = "";
        CommandElement command = edge.getCommand();
        if (command != null) {
            edgeName = command.getLabel();
        }
        return edgeName;
    }

    @NotNull
    public Collection<Edge> getEdges() {
        return edges;
    }

    @NotNull
    public String getNodeName(Node node) {
        return node.getLabel();
    }

    @NotNull
    public Collection<Node> getNodes() {
        return nodes;
    }

    @NotNull
    public Collection<Edge> edgesFrom(Node node) {
        Collection collection = new ArrayList();
        for (Edge edge : edges) {
            if (edge.getSource().equals(node)) {
                collection.add(edge);
            }
        }

        return collection;
    }

    @NotNull
    public Node getSourceNode(Edge edge) {
        return edge.getSource();
    }

    @NotNull
    public Node getTargetNode(Edge edge) {
        return edge.getTarget();
    }

    public void dispose() {

    }

    public void doLayout() {
        if (graphBuilder != null) {
            Graph2D graph = graphBuilder.getGraph();

            DirectedOrthogonalLayouter layouter = GraphSettingsProvider.getInstance(project)
                    .getSettings(graph).getDirectedOrthogonalLayouter();

            if (graph.getNodeArray().length > 0 && layouter.canLayout(graph)) {
                GraphManager graphManager = GraphManager.getGraphManager();
                graphManager.createBufferedLayouter(layouter).doLayout(graph);
                graph.updateViews();
            }
        }
    }

    public void addNode(Node node) throws DuplicatedNodeException {
        addNode(node, true);
    }

    public void addNode(Node node, boolean serialize) throws DuplicatedNodeException {
        if (propertyExists(node)) {
            // N already drawn (via createDraggedNode)
            Graph graph = graphBuilder.getGraph();
            com.intellij.openapi.graph.base.Node n =
                    GraphWrapperUtil.getInstance(project).getNode(graph, node);
            if (graph.contains(n)) {
                graph.removeNode(n);
            }

            throw new DuplicatedNodeException(node);
        } else {
            nodes.add(node);
            if (serialize) {
                update();
            }
        }
    }

    public void addElement(Node target, J2MEElement element) throws DuplicatedElementException {
        if (propertyExists(element)) {
            throw new DuplicatedElementException(element);
        } else {
            target.add(element);
            update();
        }
    }

    public void update() {
        if (graphBuilder != null) {
            graphBuilder.updateGraph();
        }
        serializer.serialize(nodes, edges);
    }

    public boolean removeNode(Node node) {
        boolean remove = nodes.remove(node);
        if (remove && graphBuilder != null) {
            removeEdges(node);
            Graph graph = graphBuilder.getGraph();
            com.intellij.openapi.graph.base.Node n =
                    GraphWrapperUtil.getInstance(project).getNode(graph, node);
            if (n != null) {
                graph.removeNode(n);
            }
            serializer.serialize(nodes, edges);
        }

        return remove;
    }

    public boolean removeEdge(Edge edge, boolean serialize) {
        boolean remove = edges.remove(edge);
        if (remove && graphBuilder != null) {
            edge.getSource().removeSource(edge);
            edge.getTarget().removeTarget(edge);
            Graph graph = graphBuilder.getGraph();
            com.intellij.openapi.graph.base.Edge e =
                    GraphWrapperUtil.getInstance(project).getEdge(graph, edge);
            if (e != null) {
                graph.removeEdge(e);
            }
            if (serialize) {
                serializer.serialize(nodes, edges);
            }
        }

        return remove;
    }

    public boolean propertyExists(J2MEElement element) {
        String elementProp = element.getProperty();
        for (Node n : nodes) {
            if (n.getProperty() != null && n.getProperty().equals(elementProp)) {
                return true;
            }
            for (J2MEElement e : n.getElements()) {
                if (e.getProperty() != null && e.getProperty().equals(elementProp)) {
                    return true;
                }
            }
        }

        return false;
    }

    public boolean propertyExists(Node node) {
        String nodeProp = node.getProperty();
        for (Node n : nodes) {
            if (n.getProperty() != null && n.getProperty().equals(nodeProp)) {
                return true;
            }
            for (J2MEElement e : n.getElements()) {
                if (e.getProperty() != null && e.getProperty().equals(nodeProp)) {
                    return true;
                }
            }
        }

        return false;
    }

    private void removeEdges(Node node) {
        removeEdges(node, null);
    }

    public void removeEdges(Node node, Edge except) {
        if (node != null) {
            for (int i = edges.size() - 1; i >= 0; i--) {
                Edge edge = edges.get(i);
                if (node.equals(edge.getSource()) || node.equals(edge.getTarget())) {
                    if (!edge.equals(except)) {
                        removeEdge(edge, false);
                    }
                }
            }
        }
    }

    public Edge associatedEdge(CommandElement command) {
        Edge edge = null;
        for (Edge e : edges) {
            CommandElement c = e.getCommand();
            if (c != null && c.equals(command)) {
                edge = e;
                break;
            }
        }
        return edge;
    }
}
