/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.ui.diagram.helper;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import net.sourceforge.intellime.J2MEBundle;
import net.sourceforge.intellime.ui.diagram.DiagramGraphDataModel;
import net.sourceforge.intellime.ui.diagram.DuplicatedNodeException;
import net.sourceforge.intellime.ui.diagram.Edge;
import net.sourceforge.intellime.ui.diagram.Node;
import net.sourceforge.intellime.ui.element.J2MEElement;
import net.sourceforge.intellime.ui.element.MIDletElement;
import net.sourceforge.intellime.ui.element.command.CommandElement;
import net.sourceforge.intellime.ui.element.screen.ScreenElement;
import net.sourceforge.intellime.util.JAXPFactory;
import net.sourceforge.intellime.util.Utils;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

public class DiagramDeserializer {

    private DiagramGraphDataModel model;
    private Project project;
    private VirtualFile file;

    private Map<String, Node> nodes = new HashMap<String, Node>();
    private Map<String, Edge> edges = new HashMap<String, Edge>();
    private Map<String, J2MEElement> elements = new HashMap<String, J2MEElement>();

    public DiagramDeserializer(DiagramGraphDataModel model) {
        this.model = model;
        this.project = model.getProject();
        this.file = model.getFile();
    }

    public void buildGraph() throws DeserializerException {
        // This is a new Diagram file
        if (Utils.isFileEmpty(project, file)) {
            MIDletElement midlet = new MIDletElement(file.getNameWithoutExtension());
            try {
                model.addNode(new Node(midlet));
            }
            catch (DuplicatedNodeException e) {
                e.printStackTrace();
            }
        }
        // In this case we need to load the xml file
        else {
            readXML();
        }
    }

    private void readXML() throws DeserializerException {
        try {
            InputStream in = file.getInputStream();

            Document doc = JAXPFactory.getInstance().getFactory().parse(in);
            NodeList nodeListPrincipal = doc.getElementsByTagName(Constants.DIAGRAM);
            NodeList nodeListApplication = nodeListPrincipal.item(0).getChildNodes();

            for (int i = 0; i < nodeListApplication.getLength(); i++) {
                org.w3c.dom.Node nodeApp = nodeListApplication.item(i);
                //this is a node element
                if (nodeApp.getNodeName().equals(Constants.NODE)) {
                    processNode(nodeApp);
                }//this is an edge element
                else if (nodeApp.getNodeName().equals(Constants.EDGE)) {
                    processEdge(nodeApp);
                }
            }
            for (Node node : nodes.values()) {
                try {
                    model.addNode(node, false);
                } catch (DuplicatedNodeException e) {
                    e.printStackTrace();
                }
            }

            for (Edge edge : edges.values()) {
                Edge modelEdge = model.createEdge(edge.getSource(), edge.getTarget(), false);
                modelEdge.setCommand(edge.getCommand());
                edge.getSource().addSource(modelEdge);
                edge.getTarget().addTarget(modelEdge);
            }

        }
        catch (SAXException e) {
            e.printStackTrace();
            throw new DeserializerException(e.getMessage());
        }
        catch (IOException e) {
            e.printStackTrace();
            throw new DeserializerException(e.getMessage());
        }
        catch (ParserConfigurationException e) {
            e.printStackTrace();
            throw new DeserializerException(e.getMessage());
        }
    }

    private void processEdge(org.w3c.dom.Node nodeApp) throws DeserializerException {
        String name = nodeApp.getAttributes().getNamedItem(Constants.NAME).getNodeValue();
        NodeList edgeList = nodeApp.getChildNodes();

        Node source = null;
        Node target = null;
        CommandElement command = null;

        for (int j = 0; j < edgeList.getLength(); j++) {

            org.w3c.dom.Node node = edgeList.item(j);

            if (Constants.SOURCE.equals(node.getNodeName())) {
                String sourceRef = node.getAttributes().getNamedItem(Constants.REF).getNodeValue();
                if (!nodes.containsKey(sourceRef)) {
                    throw new DeserializerException(J2MEBundle.message("ui.diagram.DiagramDeserializer.invalidSource"));
                }
                source = nodes.get(sourceRef);

                // retrieving command data
                command = null;
                org.w3c.dom.Node commandNode = node.getFirstChild().getNextSibling();
                if (commandNode != null) {
                    if (Constants.COMMAND.equals(commandNode.getNodeName())) {
                        String elementRef = commandNode.getAttributes().getNamedItem(Constants.ELEMENT_REF).getNodeValue();
                        if (elements.containsKey(elementRef) && elements.get(elementRef) instanceof CommandElement) {
                            command = (CommandElement) elements.get(elementRef);
                        }
                    }
                }

            } else if (Constants.TARGET.equals(node.getNodeName())) {
                String targetRef = node.getAttributes().getNamedItem(Constants.REF).getNodeValue();
                if (nodes.containsKey(targetRef)) {
                    target = nodes.get(targetRef);
                }
            }
        }

        Edge e = new Edge(source, target);
        e.setCommand(command);
        source.addSource(e);
        target.addTarget(e);

        if (e != null) {
            edges.put(name, e);
        }
    }

    private void processNode(org.w3c.dom.Node nodeApp) throws DeserializerException {
        try {
            String property = nodeApp.getAttributes().getNamedItem(Constants.PROPERTY).getNodeValue();
            String type = nodeApp.getAttributes().getNamedItem(Constants.TYPE).getNodeValue();
            String label = nodeApp.getAttributes().getNamedItem(Constants.LABEL).getNodeValue();

            Node node = null;
            ScreenElement screenElement = (ScreenElement) createElement(type, label, property);
            NodeList lstElement = nodeApp.getChildNodes();

            if (lstElement != null) {
                for (int j = 0; j < lstElement.getLength(); j++) {
                    org.w3c.dom.Node nodeElement = lstElement.item(j);
                    if (Constants.ELEMENT.equals(nodeElement.getNodeName())) {
                        String elType = nodeElement.getAttributes().getNamedItem(Constants.TYPE).getNodeValue();
                        String elLabel = nodeElement.getAttributes().getNamedItem(Constants.LABEL).getNodeValue();
                        String elProperty = nodeElement.getAttributes().getNamedItem(Constants.PROPERTY).getNodeValue();
                        J2MEElement tmpElement = createElement(elType, elLabel, elProperty);
                        screenElement.add(tmpElement);
                        elements.put(elProperty, tmpElement);
                    }
                }
            }

            node = new Node(screenElement);

            if (node != null) {
                nodes.put(property, node);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new DeserializerException(e.getMessage());
        }
    }

    private J2MEElement createElement(String type, String label, String property) throws DeserializerException {
        J2MEElement element = null;
        try {
            Class classe = Class.forName(type.trim());
            Constructor construtor;
            construtor = classe.getConstructor(new Class[]{String.class, String.class});

            if (construtor == null) {
                element = (J2MEElement) classe.newInstance();
            } else {
                element = (J2MEElement) construtor.newInstance(new Object[]{label, property});
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new DeserializerException(e.getMessage());
        }
        return element;
    }
}
