/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.ui.diagram.helper;

import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElementFactory;
import com.intellij.psi.PsiManager;
import com.intellij.psi.xml.XmlTag;
import com.intellij.util.IncorrectOperationException;
import net.sourceforge.intellime.ui.diagram.Edge;
import net.sourceforge.intellime.ui.diagram.Node;
import net.sourceforge.intellime.ui.element.J2MEElement;

import java.util.Map;

public class XmlTagFactory {

    private static XmlTagFactory xmlTagFactory;
    private PsiElementFactory factory;

    protected XmlTagFactory(Project project) {
        this.factory = PsiManager.getInstance(project).getElementFactory();
    }

    public static synchronized XmlTagFactory getInstance(Project project) {
        if (xmlTagFactory == null) {
            xmlTagFactory = new XmlTagFactory(project);
        }
        return xmlTagFactory;
    }

    public XmlTag createDiagramTag() throws IncorrectOperationException {
        return createXmlTagFromText(Constants.OPEN_TAG_DIAGRAM + Constants.CLOSE_TAG_DIAGRAM);
    }

    public XmlTag createNodeTag(Node n) throws IncorrectOperationException {

        XmlTag xmlTagNode = createXmlTagFromText(Constants.OPEN_TAG_NODE + Constants.CLOSE_TAG_NODE);
        setAttribute(xmlTagNode, Constants.ATTRIBUTE_PROPERTY, n.getProperty());
        setAttribute(xmlTagNode, Constants.ATTRIBUTE_LABEL, n.getLabel());
        setAttribute(xmlTagNode, Constants.ATTRIBUTE_TYPE, n.getScreenElement().getClass().getName());

        // Create the tags for representing node's elements
        for (J2MEElement element : n.getElements()) {
            XmlTag xmlTagElement = createXmlTagFromText(Constants.OPEN_TAG_ELEMENT + Constants.CLOSE_TAG_ELEMENT);
            setAttribute(xmlTagElement, Constants.ATTRIBUTE_PROPERTY, element.getProperty());
            setAttribute(xmlTagElement, Constants.ATTRIBUTE_LABEL, element.getLabel());
            setAttribute(xmlTagElement, Constants.ATTRIBUTE_TYPE, element.getClass().getName());
            xmlTagNode.add(xmlTagElement);
        }

        for (Map.Entry<Edge, Edge> entry : n.getLinkedEdges().entrySet()) {
            XmlTag xmlTagLinkedEdge = createXmlTagFromText(Constants.OPEN_TAG_LINKED_EDGE + Constants.CLOSE_TAG_LINKED_EDGE);
            setAttribute(xmlTagLinkedEdge, Constants.ATTRIBUTE_TARGET_EDGE, entry.getKey().toString());
            setAttribute(xmlTagLinkedEdge, Constants.ATTRIBUTE_SOURCE_EDGE, entry.getValue().toString());
            xmlTagNode.add(xmlTagLinkedEdge);
        }

        return xmlTagNode;
    }

    public XmlTag createEdgeTag(Edge e) throws IncorrectOperationException {
        XmlTag xmlTagEdge = createXmlTagFromText(Constants.OPEN_TAG_EDGE + Constants.CLOSE_TAG_EDGE);
        setAttribute(xmlTagEdge, Constants.ATTRIBUTE_NAME, e.toString());

        // Create the tag for representing the edge's source node
        XmlTag xmlTagSourceRef = createXmlTagFromText(Constants.OPEN_TAG_SOURCE + Constants.CLOSE_TAG_SOURCE);
        setAttribute(xmlTagSourceRef, Constants.ATTRIBUTE_REF, e.getSource().getProperty());

        // Create the tag for representing the source node's executed command
        XmlTag xmlTagCommandRef = createXmlTagFromText(Constants.OPEN_TAG_COMMAND + Constants.CLOSE_TAG_COMMAND);
        setAttribute(xmlTagCommandRef, Constants.ATTRIBUTE_ELEMENT_REF, e.getCommand() == null ? "" : e.getCommand().getProperty());
        xmlTagSourceRef.add(xmlTagCommandRef);

        // Create the tag for representing the edge's target node
        XmlTag xmlTagTargetRef = createXmlTagFromText(Constants.OPEN_TAG_TARGET + Constants.CLOSE_TAG_TARGET);
        setAttribute(xmlTagTargetRef, Constants.ATTRIBUTE_REF, e.getTarget().getProperty());

        xmlTagEdge.add(xmlTagSourceRef);
        xmlTagEdge.add(xmlTagTargetRef);

        return xmlTagEdge;
    }

    private XmlTag createXmlTagFromText(String text) throws IncorrectOperationException {
        return factory.createTagFromText(text);
    }

    private void setAttribute(XmlTag xmlTag, String attribute, String value) throws IncorrectOperationException {
        xmlTag.setAttribute(attribute, value);
    }

}
