/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.ui.dnd;

import com.intellij.openapi.graph.GraphManager;
import com.intellij.openapi.graph.builder.GraphBuilder;
import com.intellij.openapi.graph.builder.util.GraphWrapperUtil;
import com.intellij.openapi.graph.builder.util.NodeFactory;
import com.intellij.openapi.graph.view.HitInfo;
import com.intellij.openapi.ui.Messages;
import net.sourceforge.intellime.J2MEBundle;
import net.sourceforge.intellime.ui.diagram.DiagramGraphDataModel;
import net.sourceforge.intellime.ui.diagram.DuplicatedElementException;
import net.sourceforge.intellime.ui.diagram.DuplicatedNodeException;
import net.sourceforge.intellime.ui.diagram.Node;
import net.sourceforge.intellime.ui.diagram.renderer.J2MEElementNodeRenderer;
import net.sourceforge.intellime.ui.dialog.NewElementDialog;
import net.sourceforge.intellime.ui.dialog.NewNodeDialog;
import net.sourceforge.intellime.ui.element.J2MEElement;
import net.sourceforge.intellime.ui.element.screen.ScreenElement;

import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.*;
import java.util.List;

public class DropTargetListenerImpl implements DropTargetListener {

    private GraphBuilder builder;

    public DropTargetListenerImpl(GraphBuilder builder) {
        this.builder = builder;
    }

    private boolean isDragFlavorSupported(DropTargetDragEvent e) {
        return e.isDataFlavorSupported(J2MEElementTransferable.elementFlavor);
    }

    private DataFlavor chooseDropFlavor(DropTargetDropEvent e) {

        DataFlavor chosen = null;

        if (e.isDataFlavorSupported(J2MEElementTransferable.elementFlavor)) {
            chosen = J2MEElementTransferable.elementFlavor;
        }

        return chosen;
    }

    private boolean isDragOk(DropTargetDragEvent e) {
        boolean ok = false;

        if (isDragFlavorSupported(e)) {
            ok = true;
        }

        e.getDropAction();

        return ok;
    }

    public void dragEnter(DropTargetDragEvent e) {
        if (isDragOk(e)) {
            e.acceptDrag(e.getDropAction());
        } else {
            e.rejectDrag();
        }
    }

    public void dragOver(DropTargetDragEvent e) {
        if (isDragOk(e)) {
            e.acceptDrag(e.getDropAction());

            J2MEElementNodeRenderer.DRAG_OVER_NODE = getNode(e.getLocation());

            // update realizer
           // builder.getView().updateView(); // update all realizers

            com.intellij.openapi.graph.base.Node node = getHitInfo(e.getLocation()).getHitNode();
            if (node != null) {
                builder.getGraph().getRealizer(node).repaint(); // update current realizer
            }
            else {
                builder.getView().updateView();
            }

        } else {
            e.rejectDrag();
        }
    }

    public void dropActionChanged(DropTargetDragEvent e) {
        if (isDragOk(e)) {
            e.acceptDrag(e.getDropAction());
        } else {
            e.rejectDrag();
        }
    }

    public void dragExit(DropTargetEvent e) {

    }

    public void drop(DropTargetDropEvent e) {
        DataFlavor chosen = chooseDropFlavor(e);
        if (chosen == null) {
            e.rejectDrop();
        } else {
            Object data = null;
            try {
                e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
                data = e.getTransferable().getTransferData(chosen);

                if (data instanceof J2MEElement) {
                    if (data instanceof ScreenElement) {
                        createNode(data, e);
                    } else {
                        Node node = getNode(e.getLocation());
                        createElement(data, node);
                    }
                    e.dropComplete(true);
                } else {
                    e.dropComplete(false);
                }

                J2MEElementNodeRenderer.DRAG_OVER_NODE = null;

                // update realizer
                builder.getView().updateView(); // update all realizers
            }
            catch (Throwable t) {
                t.printStackTrace();
                e.dropComplete(false);
            }
        }
    }

    private void createNode(Object data, DropTargetDropEvent dtde) {
        ScreenElement screen = (ScreenElement) data;
        screen = (ScreenElement) screen.createCopy();
        NewNodeDialog dialog = new NewNodeDialog(screen);
        dialog.show();

        if (dialog.isOK()) {
            try {
                Node node = dialog.getObject();
                DiagramGraphDataModel model = (DiagramGraphDataModel) builder.getGraphDataModel();
                NodeFactory.getInstance().createDraggedNode(builder, node, node.getLabel(), dtde.getLocation());
                model.addNode(node);
            }
            catch (DuplicatedNodeException e) {
                Messages.showErrorDialog(
                        J2MEBundle.message("ui.diagram.DuplicatedNodeException.duplicatedNode",
                                e.getNode().getLabel() + " (" + e.getNode().getProperty() + ")"),
                        J2MEBundle.message("ui.diagram.DuplicatedNodeException.duplicatedNodeTitle"));
            }
        }
    }

    private void createElement(Object data, Node node) {
        J2MEElement element = (J2MEElement) data;
        element = element.createCopy();
        DiagramGraphDataModel model = (DiagramGraphDataModel) builder.getGraphDataModel();
        List<Node> nodes = (List<Node>) model.getNodes();
        NewElementDialog dialog = new NewElementDialog(element, nodes, node);
        dialog.show();

        if (dialog.isOK()) {
            try {
                Node target = dialog.selectedNode();
                model.addElement(target, dialog.getObject());
            }
            catch (DuplicatedElementException e) {
                Messages.showErrorDialog(
                        J2MEBundle.message("ui.diagram.DuplicatedElementException.duplicatedElement",
                                e.getElement().getLabel() + " (" + e.getElement().getProperty() + ")"),
                        J2MEBundle.message("ui.diagram.DuplicatedElementException.duplicatedElementTitle"));
            }
        }
    }

    private Node getNode(Point point) {
        Node node = null;

        HitInfo info = getHitInfo(point);

        if (info != null && info.getHitNode() != null) {
            node = GraphWrapperUtil.getInstance(builder.getProject())
                    .getNodeObject(builder.getGraph(), info.getHitNode());
        }

        return node;
    }

   private HitInfo getHitInfo(Point point) {
        double x = builder.getView().toWorldCoordX((int) point.getX());
        double y = builder.getView().toWorldCoordY((int) point.getY());
        return GraphManager.getGraphManager().createHitInfo(builder.getGraph(), x, y, false);
   }
}
