/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.ui.wizard.impl;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ProjectRootManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.PsiDirectory;
import com.intellij.psi.PsiManager;
import com.intellij.util.IncorrectOperationException;
import net.sourceforge.intellime.ui.wizard.core.WizardPage;
import net.sourceforge.intellime.ui.wizard.edit.TextEditField;
import net.sourceforge.intellime.ui.wizard.helper.GridLayoutBuilder;
import net.sourceforge.intellime.util.BeanSpector;
import net.sourceforge.intellime.util.Constants;
import net.sourceforge.intellime.util.Template;
import net.sourceforge.intellime.util.Utils;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

public class ClassWizard extends WizardImpl {

    protected static final Logger logger = Logger.getLogger(ClassWizard.class);

    protected final TextEditField base = new TextEditField();
    protected final TextEditField name = new TextEditField();
    protected final TextEditField packageField = new TextEditField();

    private String templateName;
    private String fileExtension;

    public ClassWizard(Project project, String baseClass, String packageName, String templateName, Dialog owner) {
        super(owner, project, "Class Wizard", WIZARD_ICON);
        this.templateName = templateName;
        setup(baseClass, packageName);
    }

    public ClassWizard(Project project, String baseClass, String packageName) {
        this(project, baseClass, packageName, null);
    }

    public ClassWizard(Project project, String baseClass, String packageName, String fileExtension) {
        super(project, "Class Wizard", WIZARD_ICON);

        if (fileExtension != null)
            this.fileExtension = fileExtension;
        else
            this.fileExtension = Constants.JAVA_FILE_EXTENSION;

        setup(baseClass, packageName);
    }

    protected void setup(String baseClass, String packageName) {

        packageField.setValue(packageName);
        packageField.addChangeListener(listener);

        if (this.templateName == null) {
            this.templateName = baseClass == null ? "class" : BeanSpector.getClassName(baseClass);
        }

        base.setValue(baseClass);
        base.addChangeListener(listener);
        name.addChangeListener(listener);
    }

    protected WizardPage[] createPages() {

        ArrayList<WizardPage> pages = new ArrayList<WizardPage>();
        pages.add(new WizardPage(createPage(), "New class attributes") {
            public boolean isValid() {

                return !base.isEmpty() && !name.isEmpty() && Utils.isJavaIdentifier((String) name.getValue()) && Utils.isValidPackage((String) packageField.getValue());
            }
        });

        return pages.toArray(WizardPage.EMPTY_ARRAY);
    }

    protected void checkState() {
        super.checkState();
    }

    private VirtualFile createPackage(PsiDirectory dir, String packageName) {
        VirtualFile ret;

        if (packageName != null) {
            String[] str = packageName.split("\\.");
            try {
                dir.checkCreateSubdirectory(str[0]);
                dir = dir.createSubdirectory(str[0]);
            }
            catch (IncorrectOperationException e) {
                dir = dir.findSubdirectory(str[0]);
            }

            if (packageName.indexOf('.') != -1) {
                packageName = packageName.substring(packageName.indexOf('.') + 1);
            } else {
                packageName = null;
            }

            ret = createPackage(dir, packageName);

        } else {
            return dir.getVirtualFile();
        }

        return ret;
    }

    protected Object onSuccess() {
        ApplicationManager.getApplication().runWriteAction(new Runnable() {

            public void run() {
                try {
                    create();
                }
                catch (Exception e) {
                    Messages.showErrorDialog(project, "Cannot create class", "");
                    //AppManager.getInstance().showError(project, "Cannot create class: " + ExceptionUtil.formatMessage(e));
                }
            }
        });
        return packageField.isEmpty() ? name.getValue() : packageField.getValue() + "." + name.getValue();
    }

    protected Component createPage() {
        GridLayoutBuilder builder = new GridLayoutBuilder();
        builder.addLine("Class name*", name.getComponent());
        this.defaultComponent = name.getComponent();

        builder.addLine("Base class*", base.getComponent());
        builder.addLine("Package*", packageField.getComponent());
        builder.addComponent(Box.createRigidArea(new Dimension(500, 10)));
        builder.finish();
        return builder.getPanel();
    }


    public VirtualFile create() throws Exception {

        VirtualFile[] files = ProjectRootManager.getInstance(project).getContentSourceRoots();
        PsiDirectory dir = PsiManager.getInstance(project).findDirectory(files[0]);
        VirtualFile root = null;

        if ("".equals(((String) packageField.getValue()).trim())) {
            root = dir.getVirtualFile();
        } else {
            root = createPackage(dir, (String) packageField.getValue());
        }

        String path = (String) name.getValue();
        VirtualFile file = root.createChildData(null, path + "." + fileExtension);

        if (file == null) {
            throw new Exception("Could not create " + path);
        }

        String templatePath = Constants.TEMPLATE_DIR + templateName + ".template";

        if (ClassWizard.class.getResource(templatePath) == null) {
            templatePath = Constants.CLASS_TEMPLATE;
        }
        Template template = new Template(templatePath);
        template.put("Name", path);
        template.put("Base", base.getValue());

        Object packageName = packageField.getValue();
        template.put("PACKAGE_NAME", packageName);

        try {
            template.put("extends", Class.forName((String) base.getValue()).isInterface() ? "implements" : "extends");
        } catch (ClassNotFoundException e) {
            template.put("extends", "extends");
        }
        template.evaluate(file);
        return file;
    }

}
