/*
 * Copyright 2006 IntelliME.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.util;

import net.sourceforge.intellime.ui.element.FontElement;
import net.sourceforge.intellime.ui.element.ImageElement;
import net.sourceforge.intellime.ui.element.J2MEElement;
import net.sourceforge.intellime.ui.element.TickerElement;
import net.sourceforge.intellime.ui.element.command.CommandElement;
import net.sourceforge.intellime.ui.element.item.*;
import net.sourceforge.intellime.ui.element.screen.AlertElement;
import net.sourceforge.intellime.ui.element.screen.FormElement;
import net.sourceforge.intellime.ui.element.screen.ListElement;
import net.sourceforge.intellime.ui.element.screen.TextBoxElement;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class MidletMethodFactory {

    private static MidletMethodFactory factory;

    public static synchronized MidletMethodFactory getInstance() {
        if (factory == null) {
            factory = new MidletMethodFactory();
        }
        return factory;
    }

    public String getMidletMethod(J2MEElement element) {
        String method = "";

        try {
            if (element instanceof AlertElement) {
                method = getAlertMethod((AlertElement) element);
            } else if (element instanceof FormElement) {
                method = getFormMethod((FormElement) element);
            } else if (element instanceof ListElement) {
                method = getListMethod((ListElement) element);
            } else if (element instanceof TextBoxElement) {
                method = getTextBoxMethod((TextBoxElement) element);
            } else if (element instanceof ChoiceGroup) {
                method = getChoiceGroupMethod((ChoiceGroup) element);
            } else if (element instanceof DateField) {
                method = getDateFieldMethod((DateField) element);
            } else if (element instanceof Gauge) {
                method = getGaugeMethod((Gauge) element);
            } else if (element instanceof ImageItem) {
                method = getImageItemMethod((ImageItem) element);
            } else if (element instanceof Spacer) {
                method = getSpacerMethod((Spacer) element);
            } else if (element instanceof StringItem) {
                method = getStringItemMethod((StringItem) element);
            } else if (element instanceof TextField) {
                method = getTextFieldMethod((TextField) element);
            } else if (element instanceof FontElement) {
                method = getFontElementMethod((FontElement) element);
            } else if (element instanceof ImageElement) {
                method = getImageElementMethod((ImageElement) element);
            } else if (element instanceof TickerElement) {
                method = getTickerElementMethod((TickerElement) element);
            } else if (element instanceof CommandElement) {
                method = getCommandMethod((CommandElement) element);
            }
        } catch (IOException e) {
            System.out.println(e.getMessage());
        } catch (Template.TemplateException e) {
            System.out.println(e.getMessage());
        }

        return method;
    }

    private String getAlertMethod(AlertElement alertElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.ALERT_KEY, alertElement);
        parameters.put(Constants.COMMANDS_KEY, alertElement.getCommands());
        return Utils.getTemplateEvaluateAsString(Constants.ALERT_TEMPLATE, parameters);
    }

    private String getFormMethod(FormElement formElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.FORM_KEY, formElement);
        parameters.put(Constants.COMMANDS_KEY, formElement.getCommands());
        parameters.put(Constants.ITEMS_KEY, formElement.getItems());
        return Utils.getTemplateEvaluateAsString(Constants.FORM_TEMPLATE, parameters);
    }

    private String getListMethod(ListElement listElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.LIST_KEY, listElement);
        parameters.put(Constants.COMMANDS_KEY, listElement.getCommands());
        parameters.put(Constants.NESTED_LIST_ELEMENTS_KEY, listElement.getNestedListElements());
        return Utils.getTemplateEvaluateAsString(Constants.LIST_TEMPLATE, parameters);
    }

    private String getTextBoxMethod(TextBoxElement textBoxElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.TEXT_BOX_KEY, textBoxElement);
        parameters.put(Constants.COMMANDS_KEY, textBoxElement.getCommands());
        return Utils.getTemplateEvaluateAsString(Constants.TEXT_BOX_TEMPLATE, parameters);
    }

    private String getChoiceGroupMethod(ChoiceGroup choiceGroup) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.CHOICE_GROUP_KEY, choiceGroup);
        return Utils.getTemplateEvaluateAsString(Constants.CHOICE_GROUP_TEMPLATE, parameters);
    }

    private String getDateFieldMethod(DateField dateField) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.DATE_FIELD_KEY, dateField);
        return Utils.getTemplateEvaluateAsString(Constants.DATE_FIELD_TEMPLATE, parameters);
    }

    private String getGaugeMethod(Gauge gauge) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.GAUGE_KEY, gauge);
        return Utils.getTemplateEvaluateAsString(Constants.GAUGE_TEMPLATE, parameters);
    }

    private String getImageItemMethod(ImageItem imageItem) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.IMAGE_ITEM_KEY, imageItem);
        return Utils.getTemplateEvaluateAsString(Constants.IMAGE_ITEM_TEMPLATE, parameters);
    }

    private String getSpacerMethod(Spacer spacer) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.SPACER_KEY, spacer);
        return Utils.getTemplateEvaluateAsString(Constants.SPACER_TEMPLATE, parameters);
    }

    private String getStringItemMethod(StringItem stringItem) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.STRING_ITEM_KEY, stringItem);
        return Utils.getTemplateEvaluateAsString(Constants.STRING_ITEM_TEMPLATE, parameters);
    }

    private String getTextFieldMethod(TextField textField) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.TEXT_FIELD_KEY, textField);
        return Utils.getTemplateEvaluateAsString(Constants.TEXT_FIELD_TEMPLATE, parameters);
    }

    private String getFontElementMethod(FontElement fontElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.FONT_ELEMENT_KEY, fontElement);
        return Utils.getTemplateEvaluateAsString(Constants.FONT_ELEMENT_TEMPLATE, parameters);
    }

    private String getImageElementMethod(ImageElement imageElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.IMAGE_ELEMENT_KEY, imageElement);
        return Utils.getTemplateEvaluateAsString(Constants.IMAGE_ELEMENT_TEMPLATE, parameters);
    }

    private String getTickerElementMethod(TickerElement tickerElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.TICKER_ELEMENT_KEY, tickerElement);
        return Utils.getTemplateEvaluateAsString(Constants.TICKER_ELEMENT_TEMPLATE, parameters);
    }

    private String getCommandMethod(CommandElement commandElement) throws IOException, Template.TemplateException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put(Constants.COMMAND_KEY, commandElement);
        return Utils.getTemplateEvaluateAsString(Constants.COMMAND_TEMPLATE, parameters);
    }

}
