/*
 * Copyright 2000-2006 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.intellime.util;

import com.intellij.openapi.vfs.VirtualFile;
import org.apache.log4j.Logger;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.VelocityException;
import org.jetbrains.annotations.NonNls;

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Date: 14.04.2005 Time: 12:37:50
 *
 * @author Dmitry Avdeev
 */
public class Template {

    public static class TemplateException extends Exception {
        TemplateException(Exception e) {
            super(e.getMessage(), e);
        }
    }

    protected final static Logger logger = Logger.getLogger(Template.class);

    private final static DateFormat DATE_FORMAT = new SimpleDateFormat("dd.MM.yyyy");
    private final static DateFormat TIME_FORMAT = new SimpleDateFormat("hh:mm:ss");

    private final String fileName;
    private final VelocityContext context = new VelocityContext();

    private final static VelocityEngine velocity;

    static {
        velocity = new VelocityEngine();
        try {
            velocity.init();
        }
        catch (Exception e) {
            logger.error(e);
        }
    }

    public Template(@NonNls String fileName) {
        this.fileName = fileName;
        Date now = new Date();
        put("DATE", DATE_FORMAT.format(now));
        put("TIME", TIME_FORMAT.format(now));
    }

    public void put(@NonNls String key, Object value) {
        context.put(key, value);
    }

    public void evaluate(Writer writer) throws IOException, TemplateException {

        InputStream in = Template.class.getResourceAsStream(fileName);
        if (in == null) {
            throw new FileNotFoundException(fileName);
        }
        try {
            velocity.evaluate(context, writer, "out", new InputStreamReader(in));
            writer.flush();
        }
        catch (VelocityException e) {
            throw new TemplateException(e);
        }
        finally {
            in.close();
        }
    }

    public void evaluate(VirtualFile target) throws IOException, TemplateException {
        Writer writer = new OutputStreamWriter(target.getOutputStream(context, -1, -1));
        try {
            evaluate(writer);
            writer.flush();
        }
        finally {
            writer.close();
        }

    }
}
